from django import forms
from django.forms.widgets import Textarea


class TinyMceEditorWidget(Textarea):
    """
    Widget providing TinyMce for Rich Text Editing.
    """
    @property
    def media(self):
        return forms.Media(
            js=(
                'tinymce/tinymce.min.js',
                'tinymce/tinymce.init.js',
            ),
        )

    def __init__(self, attrs=None):
        default_attrs = {'data-editor': 'tinymce'}
        if attrs:
            default_attrs.update(attrs)
        super(TinyMceEditorWidget, self).__init__(default_attrs)


class RichTextEditorWidget(TinyMceEditorWidget):
    pass



from django.shortcuts import Http404
from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django_admin_kubi.utils import get_kubi_feature


def get_searchable_models(request):
    """ Returns (model, perm,) for all models user can possibly see """
    items = []
    admin_site = admin.site

    for model, model_admin in admin_site._registry.items():
        search_fields = model_admin.get_search_fields(request)
        if not search_fields:
            continue

        # check permissions
        perms = model_admin.get_model_perms(request)
        if 'add' in perms:
            del perms['add']
        if True not in perms.values():
            continue
        items.append((model, model_admin, perms,))
    return items


def get_model_url_by_perm(opts, perms):
    info = opts.app_label, opts.model_name
    if 'change' in perms and perms['change']:
        return ('change', 'admin:%s_%s_change' % info)
    if 'delete' in perms and perms['delete']:
        return ('delete', 'admin:%s_%s_delete' % info)
    return None


def search(request):
    user = request.user

    if not request.method == "GET":
        raise Http404()

    if not (user.is_staff or user.is_superadmin):
        raise Http404()

    if not get_kubi_feature('ADMIN_SEARCH'):
        raise PermissionDenied

    search_term = request.GET.get("q", None)
    results = list()
    total_results = 0
    if search_term and len(search_term) >= 2:
        for model, model_admin, perms in get_searchable_models(request):
            queryset = model_admin.get_queryset(request)
            result = model_admin.get_search_results(request, queryset, search_term)[0]
            opts = model._meta
            if result:
                total_results += len(result)
                results.append({
                    'results': result,
                    'opts': opts,
                    'url': get_model_url_by_perm(opts, perms)
                })

    context = {
        'title': _('Search'),
        'term': search_term,
        'results': results,
        'total_results': total_results
    }
    return TemplateResponse(request, 'admin/search.html', context)


def history(request):
    from django_admin_kubi.admin import HistoryLogAdmin
    from django.contrib.admin.models import LogEntry

    user = request.user
    if not user.is_staff:
        raise Http404()

    if get_kubi_feature('ADMIN_HISTORY'):
        history_admin = HistoryLogAdmin(LogEntry, admin.site)
        return history_admin.changelist_view(request)

    raise PermissionDenied



from __future__ import unicode_literals

from fnmatch import fnmatch
from collections import OrderedDict

from django.contrib import admin
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_admin_kubi.admin_menu.items import *
import six

# by default exclude all auth models
DEFAULT_EXCLUDE_MODELS = ("django.contrib.auth.*", )


class MenuBase(type):
    def __new__(cls, name, bases, attrs):
        super_new = super(MenuBase, cls).__new__

        # Also ensure initialization is only performed for subclasses of Menu
        # (excluding Model class itself).
        parents = [b for b in bases if isinstance(b, MenuBase)]
        if not parents:
            return super_new(cls, name, bases, attrs)

        current_items = []
        for key, value in list(attrs.items()):
            if isinstance(value, MenuItem):
                current_items.append((key, value))
                attrs.pop(key)
        current_items.sort(key=lambda x: x[1].creation_counter)

        attrs['declared_items'] = OrderedDict(current_items)

        new_class = super_new(cls, name, bases, attrs)

        return new_class


class Menu(six.with_metaclass(MenuBase)):
    children = None
    context = None
    available_models = None

    def __init__(self, **kwargs):
        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])
        self.children = kwargs.get('children', [])

        for key, item in self.declared_items.items():
            item.set_parent(self)

    def __iter__(self):
        for key, item in self.declared_items.items():
            yield item

    def get_admin_site(self):
        return admin.site

    def get_admin_site_name(self):
        return self.get_admin_site().name

    def get_avail_models(self, request):
        """ Returns (model, perm,) for all models user can possibly see """
        items = []
        admin_site = admin.site

        for model, model_admin in admin_site._registry.items():
            perms = model_admin.get_model_perms(request)
            if True not in perms.values():
                continue
            items.append((model, perms,))
        return items

    def get_model(self, pattern):
        items = self.available_models
        full_name = lambda model: '%s.%s' % (model.__module__, model.__name__)

        for item in items:
            model, perms = item
            if full_name(model) == pattern:
                return item

        return None

    def filter_models(self, models, exclude):
        """
        Returns (model, perm,) for all models that match models/exclude patterns
        and are visible by current user.
        """
        items = self.available_models
        included = []
        full_name = lambda model: '%s.%s' % (model.__module__, model.__name__)

        if len(models) == 0:
            included = items
        else:
            for pattern in models:
                for item in items:
                    model, perms = item
                    if fnmatch(full_name(model), pattern) and item not in included:
                        included.append(item)

        result = included[:]
        for pattern in exclude:
            for item in included:
                model, perms = item
                if fnmatch(full_name(model), pattern):
                    result.remove(item)
        return result

    def set_context(self, context):
        self.context = context
        request = context.get('request')
        if not request:
            raise ImproperlyConfigured("Missing Request Processor")
        self.available_models = self.get_avail_models(request)

    def _get_admin_app_list_url(self, model):
        """
        Returns the admin change url.
        """
        app_label = model._meta.app_label
        return reverse('%s:app_list' % self.get_admin_site_name(), args=(app_label,))

    def _get_admin_change_url(self, model):
        """
        Returns the admin change url.
        """
        app_label = model._meta.app_label
        return reverse('%s:%s_%s_changelist' % (self.get_admin_site_name(),
                                                app_label,
                                                model.__name__.lower()))

    def _get_admin_add_url(self, model):
        """
        Returns the admin add url.
        """
        app_label = model._meta.app_label
        return reverse('%s:%s_%s_add' % (self.get_admin_site_name(),
                                         app_label,
                                         model.__name__.lower()))


class AdminMenu(Menu):
    dashboard = MenuItem(_('Dashboard'), reverse('admin:index'), icon="fa-th-large")
    applications = AppList(_('Applications'), models=("*"), exclude=DEFAULT_EXCLUDE_MODELS, icon="fa-laptop")
    administration = ModelList(_('Administration'), models=DEFAULT_EXCLUDE_MODELS, icon="fa-cogs")



from functools import total_ordering

from django.conf import settings
from django.utils.text import capfirst
from django.template.loader import render_to_string


def get_model_icon_overwrites(model):
    MODEL_NAME_ICON_MAP = {
        "User": "fa-user",
        "Group": "fa-users",
        "Tag": "fa-tags",
        "Site": "fa-globe",
        "LogEntry": "fa-clipboard-list",
        "Watermark": "fa-marker",
        "PhotoEffect": "fa-spray-can",
        "PhotoSize": "fa-crop-alt",
        "Photo": "fa-image",
        "Gallery": "fa-images",

    }
    model_name = model.__name__
    if hasattr(settings, "MODEL_NAME_ICON_MAP"):
        MODEL_NAME_ICON_MAP = settings.MODEL_NAME_ICON_MAP
    return MODEL_NAME_ICON_MAP.get(model_name, None)


@total_ordering
class MenuItem(object):
    menu = None
    title = 'Menu item'
    url = '#'
    css_classes = None
    enabled = True
    children = None
    icon = None
    parent = None
    creation_counter = 0

    def __eq__(self, other):
        # Needed for @total_ordering
        if isinstance(other, MenuItem):
            return self.creation_counter == other.creation_counter
        return NotImplemented

    def __lt__(self, other):
        # This is needed because bisect does not take a comparison function.
        if isinstance(other, MenuItem):
            return self.creation_counter < other.creation_counter
        return NotImplemented

    def __hash__(self):
        return hash(self.creation_counter)

    def __init__(self, title, url=None, **kwargs):
        self.title = title

        if url is not None:
            self.url = url

        self.children = self.children or []
        self.icon = kwargs.get('icon', None)

        self.creation_counter = MenuItem.creation_counter
        MenuItem.creation_counter += 1

        # limit creation_counter to 1000, menus should not be this big
        if MenuItem.creation_counter > 1000:
            MenuItem.creation_counter = 0

    def render(self):
        if not self.context():
            return ""
        if self.url == "#" and not self.children:
            return ""

        self.set_parent_to_children()

        context = {
            'icon': self.icon,
            'title': self.title,
            'url': self.url,
            'children': self.children,
            'is_selected': self.is_selected(),
            'parent': self.parent,
        }
        return render_to_string('admin/sidebar_item.html', context)

    def context(self):
        return True

    def set_parent(self, parent):
        if isinstance(parent, MenuItem):
            self.parent = parent
            self.menu = self.parent.menu
        else:
            self.parent = None
            self.menu = parent

    def set_parent_to_children(self):
        for child in self.children:
            child.set_parent(self)

    def is_selected(self, request=None):
        if not request and self.menu:
            request = self.menu.context.get("request")
        if request:
            current_url = request.get_full_path()
            return self.url == current_url or \
                len([c for c in self.children if c.is_selected(request)]) > 0

        return False


class ModelUrl(MenuItem):
    model = None

    def __init__(self, title, model, **kwargs):
        self.model = model
        super(ModelUrl, self).__init__(title, **kwargs)

    def context(self):
        item = self.menu.get_model(self.model)
        if item:
            model, perms = item
            if perms['view']:
                if hasattr(model, 'Menu'):
                    if hasattr(model.Menu, 'icon'):
                        self.icon = model.Menu.icon

                self.title = capfirst(model._meta.verbose_name_plural)
                self.url = self.menu._get_admin_change_url(model)

                return True

        return False


class ModelItem(MenuItem):
    model = None

    def __init__(self, model, title=None, **kwargs):
        self.model = model
        super(ModelItem, self).__init__(title, **kwargs)

    def context(self):
        item = self.menu.get_model(self.model)
        if item:
            model, perms = item
            if perms['view']:
                if hasattr(model, 'Menu'):
                    if hasattr(model.Menu, 'icon'):
                        self.icon = model.Menu.icon

                self.title = capfirst(model._meta.verbose_name_plural)
                self.url = self.menu._get_admin_change_url(model)

                return True

        return False


class AppList(MenuItem):
    def __init__(self, title=None, **kwargs):
        self.models = list(kwargs.pop('models', []))
        self.exclude = list(kwargs.pop('exclude', []))
        super(AppList, self).__init__(title, **kwargs)

    def context(self):
        self.children = []
        items = self.menu.filter_models(self.models[:], self.exclude[:])
        apps = {}
        for model, perms in items:
            if not perms['view']:
                continue

            icon = None
            title = capfirst(model._meta.verbose_name_plural)

            app_label = model._meta.app_config.verbose_name
            if app_label not in apps:
                apps[app_label] = {
                    'title': capfirst(app_label.title()),
                    'icon': icon,
                    'url': self.menu._get_admin_app_list_url(model),
                    'models': []
                }

            if hasattr(model, 'Menu'):
                if hasattr(model.Menu, 'title'):
                    title = model.Menu.title
                if hasattr(model.Menu, 'icon'):
                    icon = model.Menu.icon

            apps[app_label]['models'].append({
                'icon': icon,
                'title': title,
                'url': self.menu._get_admin_change_url(model)
            })

        # sort key names
        apps_sorted = list(apps.keys())
        apps_sorted.sort()

        for app in apps_sorted:
            app_dict = apps[app]
            item = MenuItem(title=app_dict['title'], url=app_dict['url'], icon=app_dict['icon'])

            for model_dict in apps[app]['models']:
                item.children.append(MenuItem(**model_dict))
            self.children.append(item)

        return True


class ModelList(MenuItem):
    def __init__(self, title, models, exclude=None, **kwargs):
        self.models = list(models or [])
        self.exclude = list(exclude or [])
        super(ModelList, self).__init__(title, **kwargs)

    def context(self):
        self.children = []
        items = self.menu.filter_models(self.models[:], self.exclude[:])

        for model, perms in items:
            if not perms['view']:
                continue

            title = capfirst(model._meta.verbose_name_plural)
            icon = get_model_icon_overwrites(model)
            if hasattr(model, 'Menu'):
                if hasattr(model.Menu, 'title'):
                    title = model.Menu.title
                if hasattr(model.Menu, 'icon'):
                    icon = model.Menu.icon

            url = self.menu._get_admin_change_url(model)
            item = MenuItem(title=title, url=url, icon=icon)
            self.children.append(item)

        return True



#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

setup(
    name='django-admin-kubi',
    version='1.0.1',
    description='A Django admin template replacement focused on flexibility and user experience',
    author='Rafael Fernandes',
    author_email='rafael@dengun.com',
    url='https://github.com/dengunorg/django-admin-kubi',
    packages=find_packages(),
    include_package_data=True,
    license='BSD',
    long_description=open('description.rst').read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    requires=['Django(>=4.0)'],
    install_requires=[],
    zip_safe=False,
)



from __future__ import unicode_literals

import hashlib
import urllib
import re

from django import template
from django.db import models
from django.http import QueryDict
from django.utils import formats
from django.conf import settings
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.text import Truncator, capfirst
from django.utils.translation import gettext_lazy as _
from django.utils.module_loading import import_string
from django.template import defaultfilters
from django.contrib.admin.views.main import PAGE_VAR
from django_admin_kubi.admin_menu.items import get_model_icon_overwrites
from django_admin_kubi.utils import get_kubi_feature
from django.utils.timezone import is_aware
import datetime

from django.utils.translation import (
    gettext_lazy,
    ngettext,
    ngettext_lazy,
    npgettext_lazy,
    pgettext,
    round_away_from_one,
)

register = template.Library()


@register.simple_tag()
def has_kubi_feature(feature_name):
    return get_kubi_feature(feature_name)


def get_admin_menu():
    """
    Returns the admin menu defined by the user or the default one.
    """
    menu_cls = getattr(
        settings,
        'ADMIN_MENU',
        'django_admin_kubi.admin_menu.menu.AdminMenu'
    )

    MENU = import_string(menu_cls)
    return MENU()


@register.inclusion_tag('admin/sidebar_menu.html', takes_context=True)
def admin_menu(context):

    menu = get_admin_menu()
    menu.set_context(context)

    # RENDER MENU
    context.update({
        'menu': menu
    })
    return context


def silence_without_field(fn):
    def wrapped(field, attr):
        if not field:
            return ""
        else:
            return fn(field, attr)
    return wrapped


def _process_field_attributes(field, attr, process):
    # split attribute name and value from 'attr:value' string
    params = attr.split(':', 1)
    attribute = params[0]
    value = params[1] if len(params) == 2 else ''

    # decorate field.as_widget method with updated attributes
    old_as_widget = field.as_widget

    def as_widget(self, widget=None, attrs=None, only_initial=False):
        attrs = attrs or {}
        process(widget or self.field.widget, attrs, attribute, value)
        return old_as_widget(widget, attrs, only_initial)

    bound_method = type(old_as_widget)
    try:
        field.as_widget = bound_method(as_widget, field, field.__class__)
    except TypeError:
        field.as_widget = bound_method(as_widget, field)
    return field


@register.filter("append_attr")
@silence_without_field
def append_attr(field, attr):
    def process(widget, attrs, attribute, value):
        if attrs.get(attribute):
            attrs[attribute] += ' ' + value
        elif widget.attrs.get(attribute):
            attrs[attribute] = widget.attrs[attribute] + ' ' + value
        else:
            attrs[attribute] = value
    return _process_field_attributes(field, attr, process)


@register.filter("label_tag")
def label_tag(field, css_class=None):
    if css_class is None:
        css_class = ''
    output = ""
    if isinstance(field, dict):
        output = u'<label class="form-label %s">%s:</label>' % (css_class, field.label)
    else:
        if field.field.required:
            output = u'<label for="%s" class="form-label %s required">%s: <small class="text-required">*</small></label>' % (field.id_for_label, css_class, field.label)
        else:
            output = u'<label for="%s" class="form-label %s">%s:</label>' % (field.id_for_label, css_class, field.label)

    return mark_safe(output)


@register.filter("line_class")
def line_class(fields):
    l = len(fields)
    if l > 1:
        size = int(12/l)
        if size < 1:
            size = 1
        return "col-md-%s col-lg-%s" % (size, size)
    return ""


@register.filter("add_class")
@silence_without_field
def add_class(field, css_class):
    return append_attr(field, 'class:' + css_class)


@register.filter("add_placeholder")
@silence_without_field
def add_placeholder(field, placeholder_value):
    return append_attr(field, 'placeholder:' + placeholder_value)


@register.filter(name='widget_type')
def widget_type(field):
    """
    Template filter that returns field widget class name (in lower case).
    E.g. if field's widget is TextInput then {{ field|widget_type }} will
    return 'textinput'.
    """
    if hasattr(field, 'field') and hasattr(field.field, 'widget') and field.field.widget:
        if hasattr(field.field.widget, 'widget') and field.field.widget.widget:
            return field.field.widget.widget.__class__.__name__.lower()
        return field.field.widget.__class__.__name__.lower()
    return ''


@register.filter
def gravatar_url(email, size=40):
    """
    returns gravatar url based on provided email
    Usage: {{ email|gravatar_url:150 }}
    """
    default = "identicon"
    return "https://www.gravatar.com/avatar/%s?%s" % (hashlib.md5(email.lower().encode('utf-8')).hexdigest(), urllib.parse.urlencode({'d': default, 's': str(size)}))


# return an image tag with the gravatar
# TEMPLATE USE:  {{ email|gravatar:150 }}
@register.filter
def gravatar(email, size=40):
    url = gravatar_url(email, size)
    return mark_safe('<img src="%s">' % url)


@register.filter
def user_avatar(user):
    initials = None
    if user.first_name and user.last_name:
        initials = "%s%s" % (user.first_name[0], user.last_name[0])
    elif user.first_name:
        initials = "%s%s" % (user.first_name[0], user.first_name[1])
    elif user.last_name:
        initials = "%s%s" % (user.last_name[0], user.last_name[1])

    if user.email:
        image = gravatar(user.email, size=60)
        output = '<div class="avatar gravatar">%s</div>' % image
    elif initials:
        output = '<div class="avatar initials">%s</div>' % initials.upper()
    else:
        output = '<div class="avatar icon"><i class="fa fa-leaf"></i></div>'

    return mark_safe(output)


@register.filter
def user_short_name(user):
    output = []
    if user.first_name:
        output += re.sub("[^\w]", " ",  user.first_name).split()
    if user.last_name:
        output += re.sub("[^\w]", " ",  user.last_name).split()

    if not output:
        output = [user.username if hasattr(user, 'username') else user.email]

    try:
        first_part = output[:1][0]
        second_part = output[-1:][0]
        if first_part == second_part:
            output = first_part
        else:
            output = "%s %s." % (first_part, second_part[0])
    except:
        output = user.username if hasattr(user, 'username') else user.email

    return mark_safe(output)


@register.filter("user_type")
def user_type(user):
    if user.is_superuser:
        output = _('Administrator')
    elif user.is_staff:
        output = _('Staff')
    else:
        output = _('User')

    return mark_safe(output)


@register.filter
def user_admin_urlname(value, arg):
    """
    Providing a link to change the user
    (for working better with auth custom user)
    """
    return 'admin:%s_%s_%s' % (value._meta.app_label, value._meta.model_name, arg)


@register.filter(is_safe=True)
@defaultfilters.stringfilter
def truncate_title(value):
    """
    Truncates the title to a specific char size.
    """
    return Truncator(value).chars(32)


@register.simple_tag
def breadcrumbs_icon(options):
    """
    Returns a font awesome icon to be used in the breadcrumbs block
    """
    model = None
    icon = "fa-default"
    if isinstance(options, models.Model):
        model = options
    elif hasattr(options, 'model'):
        model = options.model
    elif isinstance(options, str):
        icon = options
    if model and get_model_icon_overwrites(model):
        icon = get_model_icon_overwrites(model)
    if model and hasattr(model, 'Menu'):
        if hasattr(model.Menu, 'icon'):
            icon = model.Menu.icon
    return mark_safe('<span class="breadcrumb-icon"><i class="fa %s"></i></span>' % icon)


@register.simple_tag
def paginator_number(cl, i):
    """
    Generates an individual page index link in a paginated list.
    """
    if i == '.':
        return format_html(u'<li class="page-item disabled"><span class="page-link">...</span></li>')
    elif i == cl.page_num:
        return format_html(u'<li class="page-item active"><span class="page-link">{0}</span></li> ', i)
    else:
        return format_html(u'<li class="page-item"><a class="page-link" href="{0}"{1}>{2}</a></li>', cl.get_query_string({PAGE_VAR: i}), mark_safe(' class="end"' if i == cl.paginator.num_pages-1 else ''), i)


class AdminLogNode(template.Node):
    """
        NOTE: I've changed the original behaviour in order to provide a
        Admin Log for a specific app on the AdminSite.app_index.
    """
    def __init__(self, limit, varname, user):
        self.limit, self.varname, self.user = limit, varname, user

    def __repr__(self):
        return "<GetAdminLog Node>"

    def render(self, context):
        from django.contrib.admin.models import LogEntry

        app_label = context.get('app_label')
        if app_label:
            if self.user is None:
                context[self.varname] = LogEntry.objects.filter(content_type__app_label__exact=app_label).select_related('content_type', 'user')[:self.limit]
            else:
                user_id = self.user
                if not user_id.isdigit():
                    user_id = context[self.user].id
                context[self.varname] = LogEntry.objects.filter(user__id__exact=user_id, content_type__app_label__exact=app_label).select_related('content_type', 'user')[:int(self.limit)]
        return ''


@register.tag
def get_admin_log_for_app(parser, token):
    """
    NOTE: I've changed the original behaviour in order to provide a
    Admin Log for a specific app on the AdminSite.app_index.

    Populates a template variable with the admin log for the given criteria.

    Usage::

        {% get_admin_log_for_app [limit] as [varname] for_user [context_var_containing_user_obj] %}

    Examples::

        {% get_admin_log_for_app 10 as admin_log for_user 23 %}
        {% get_admin_log_for_app 10 as admin_log for_user user %}
        {% get_admin_log_for_app 10 as admin_log %}

    Note that ``context_var_containing_user_obj`` can be a hard-coded integer
    (user ID) or the name of a template context variable containing the user
    object whose ID you want.
    """
    tokens = token.contents.split()
    if len(tokens) < 4:
        raise template.TemplateSyntaxError(
            "'get_admin_log_for_app' statements require two arguments")
    if not tokens[1].isdigit():
        raise template.TemplateSyntaxError(
            "First argument to 'get_admin_log_for_app' must be an integer")
    if tokens[2] != 'as':
        raise template.TemplateSyntaxError(
            "Second argument to 'get_admin_log_for_app' must be 'as'")
    if len(tokens) > 4:
        if tokens[4] != 'for_user':
            raise template.TemplateSyntaxError(
                "Fourth argument to 'get_admin_log_for_app' must be 'for_user'")
    return AdminLogNode(limit=tokens[1], varname=tokens[3], user=(len(tokens) > 5 and tokens[5] or None))


@register.simple_tag(takes_context=True)
def get_fieldsets_and_inlines(context):
    """
    USAGE:
    In ModelAdmin()
    -----------------------------
    (i | inline)   :Show inline
    (f | fieldset) :Show filedset
    -----------------------------
    eg:
    list_fieldsets_inlines_order = ('i', 'inline', 'i', 'fielset', 'f')
    """

    adminform = context['adminform']
    model_admin = adminform.model_admin
    adminform = iter(adminform)

    # Check if has inlines, can be a custom admin view
    inlines = iter([])
    if 'inline_admin_formsets' in context:
        inlines = iter(context['inline_admin_formsets'])

    obj = None
    if 'original'in context:
        obj = context['original']

    request = QueryDict()
    if 'request'in context:
        request = context['request']

    # Get Order
    list_fieldsets_inlines_order = ()
    if hasattr(model_admin, 'list_fieldsets_inlines_order'):
        list_fieldsets_inlines_order = model_admin.list_fieldsets_inlines_order

    if hasattr(model_admin, 'get_list_fieldsets_inlines_order'):
        list_fieldsets_inlines_order = model_admin.get_list_fieldsets_inlines_order(request, obj=obj)

    # Create MIX
    fieldsets_and_inlines = []
    for choice in list_fieldsets_inlines_order:

        if choice in ('f', 'fieldset'):

            try:
                fieldsets_and_inlines.append(('fieldset', adminform.__next__()))
            except StopIteration:
                pass

        elif choice in ('i', 'inline'):

            try:
                fieldsets_and_inlines.append(('inline', inlines.__next__()))
            except StopIteration:
                pass

    for fieldset in adminform:
        fieldsets_and_inlines.append(('fieldset', fieldset))

    for inline in inlines:
        fieldsets_and_inlines.append(('inline', inline))

    return fieldsets_and_inlines


@register.inclusion_tag('admin/date_hierarchy.html')
def date_hierarchy(cl):
    """
    Displays the date hierarchy for date drill-down functionality.
    """
    if cl.date_hierarchy:
        field_name = cl.date_hierarchy
        year_field = '%s__year' % field_name
        month_field = '%s__month' % field_name
        day_field = '%s__day' % field_name
        field_generic = '%s__' % field_name
        year_lookup = cl.params.get(year_field)
        month_lookup = cl.params.get(month_field)
        day_lookup = cl.params.get(day_field)
        one_year_only = False
        one_month_only = False

        def link(filters):
            return cl.get_query_string(filters, [field_generic])

        if not (year_lookup or month_lookup or day_lookup):
            # select appropriate start level
            date_range = cl.queryset.aggregate(first=models.Min(field_name),
                                               last=models.Max(field_name))
            if date_range['first'] and date_range['last']:
                if date_range['first'].year == date_range['last'].year:
                    year_lookup = date_range['first'].year
                    one_year_only = True
                    if date_range['first'].month == date_range['last'].month:
                        month_lookup = date_range['first'].month
                        one_month_only = True

        if year_lookup and month_lookup and day_lookup:
            day = datetime.date(int(year_lookup), int(month_lookup), int(day_lookup))
            return {
                'show': True,
                'back': {
                    'link': link({year_field: year_lookup, month_field: month_lookup}),
                    'title': capfirst(formats.date_format(day, 'YEAR_MONTH_FORMAT'))
                },
                'choice': capfirst(formats.date_format(day)),
            }
        elif year_lookup and month_lookup:
            days = getattr(cl.queryset, 'dates')(field_name, 'day')
            cday = datetime.date(int(year_lookup), int(month_lookup), int(1))
            return {
                'show': True,
                'back': {
                    'link': link({year_field: year_lookup}),
                    'title': str(year_lookup)
                } if not one_month_only else {},
                'choice': capfirst(formats.date_format(cday, 'YEAR_MONTH_FORMAT')),
                'choices': [{
                    'link': link({year_field: year_lookup, month_field: month_lookup, day_field: day.day}),
                    'title': capfirst(formats.date_format(day, 'j - l'))
                } for day in days]
            }
        elif year_lookup:
            months = getattr(cl.queryset, 'dates')(field_name, 'month')
            return {
                'show': True,
                'back': {
                    'link': link({}),
                    'title': _('All dates')
                } if not one_year_only else {},
                'choice': str(year_lookup),
                'choices': [{
                    'link': link({year_field: year_lookup, month_field: month.month}),
                    'title': capfirst(formats.date_format(month, 'F'))
                } for month in months]
            }
        else:
            years = getattr(cl.queryset, 'dates')(field_name, 'year')
            return {
                'show': len(years) > 0,
                'choices': [{
                    'link': link({year_field: str(year.year)}),
                    'title': str(year.year),
                } for year in years]
            }


@register.filter
def naturaltimeshort(value):
    return NaturalTimeShortFormatter.string_for(value)


class NaturalTimeShortFormatter:
    time_strings = {
        # Translators: delta will contain a string like '2 months' or '1 month, 2 weeks'
        "past-day": gettext_lazy("%(delta)s"),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "past-hour": ngettext_lazy("1h", "%(count)sh", "count"),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "past-minute": ngettext_lazy("1m", "%(count)sm", "count"),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "past-second": ngettext_lazy("1s", "%(count)ss", "count"),
        "now": gettext_lazy("now"),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "future-second": ngettext_lazy(
            "in 1s", "in %(count)ss", "count"
        ),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "future-minute": ngettext_lazy(
            "in 1m", "in %(count)sm", "count"
        ),
        # Translators: please keep a non-breaking space (U+00A0) between count
        # and time unit.
        "future-hour": ngettext_lazy(
            "in 1h", "in %(count)sh", "count"
        ),
        # Translators: delta will contain a string like '2 months' or '1 month, 2 weeks'
        "future-day": gettext_lazy("in %(delta)s"),
    }
    past_substrings = {
        # Translators: 'naturaltime-past' strings will be included in '%(delta)s ago'
        "year": npgettext_lazy(
            "naturaltimeshort-past", "%(num)dy", "%(num)d y", "num"
        ),
        "month": npgettext_lazy(
            "naturaltimeshort-past", "%(num)dmo", "%(num)dmo", "num"
        ),
        "week": npgettext_lazy(
            "naturaltimeshort-past", "%(num)dw", "%(num)dw", "num"
        ),
        "day": npgettext_lazy("naturaltime-past", "%(num)dd", "%(num)dd", "num"),
        "hour": npgettext_lazy(
            "naturaltimeshort-past", "%(num)dh", "%(num)dh", "num"
        ),
        "minute": npgettext_lazy(
            "naturaltimeshort-past", "%(num)dm", "%(num)dm", "num"
        ),
    }
    future_substrings = {
        # Translators: 'naturaltime-future' strings will be included in
        # '%(delta)s from now'.
        "year": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dy", "%(num)dy", "num"
        ),
        "month": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dmo", "%(num)dmo", "num"
        ),
        "week": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dw", "%(num)dw", "num"
        ),
        "day": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dd", "%(num)dd", "num"
        ),
        "hour": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dh", "%(num)dh", "num"
        ),
        "minute": npgettext_lazy(
            "naturaltimeshort-future", "%(num)dm", "%(num)dm", "num"
        ),
    }

    @classmethod
    def string_for(cls, value):
        if not isinstance(value, datetime.date):  # datetime is a subclass of date
            return value

        now = datetime.datetime.now(datetime.timezone.utc if is_aware(value) else None)
        if value < now:
            delta = now - value
            if delta.days != 0:
                return cls.time_strings["past-day"] % {
                    "delta": defaultfilters.timesince(
                        value, now, time_strings=cls.past_substrings
                    ),
                }
            elif delta.seconds == 0:
                return cls.time_strings["now"]
            elif delta.seconds < 60:
                return cls.time_strings["past-second"] % {"count": delta.seconds}
            elif delta.seconds // 60 < 60:
                count = delta.seconds // 60
                return cls.time_strings["past-minute"] % {"count": count}
            else:
                count = delta.seconds // 60 // 60
                return cls.time_strings["past-hour"] % {"count": count}
        else:
            delta = value - now
            if delta.days != 0:
                return cls.time_strings["future-day"] % {
                    "delta": defaultfilters.timeuntil(
                        value, now, time_strings=cls.future_substrings
                    ),
                }
            elif delta.seconds == 0:
                return cls.time_strings["now"]
            elif delta.seconds < 60:
                return cls.time_strings["future-second"] % {"count": delta.seconds}
            elif delta.seconds // 60 < 60:
                count = delta.seconds // 60
                return cls.time_strings["future-minute"] % {"count": count}
            else:
                count = delta.seconds // 60 // 60
                return cls.time_strings["future-hour"] % {"count": count}



from django.contrib import admin
from django.urls import reverse
from django.core.exceptions import PermissionDenied
from django.template.defaultfilters import capfirst
from django.utils.translation import gettext_lazy as _
from django.utils.safestring import mark_safe
from functools import wraps


class HistoryLogActionFilter(admin.SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _('Action')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'flag'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('1', _('Addition')),
            ('2', _('Change')),
            ('3', _('Deletion')),
        )

    def queryset(self, request, queryset):
        if self.value() == '1':
            return queryset.filter(action_flag=1)
        if self.value() == '2':
            return queryset.filter(action_flag=2)
        if self.value() == '3':
            return queryset.filter(action_flag=3)


class HistoryLogAdmin(admin.ModelAdmin):
    actions = None
    list_per_page = 25
    list_display = ('action_time', 'user', 'content_display', 'repr_display', 'message_display')
    list_filter = ('user', 'content_type', HistoryLogActionFilter)
    search_fields = ('user__username', 'change_message')
    list_select_related = ('content_type', 'user',)
    list_max_show_all = False

    def message_display(self, obj):
        message = obj.get_change_message()
        if message:
            return message
        return '...'

    message_display.short_description = _('Message')

    def repr_display(self, obj):
        if obj.is_addition():
            icon = '<i class="fa fa-plus"></i>'
            cssclass = 'text-success'
        elif obj.is_change():
            icon = '<i class="fa fa-edit"></i>'
            cssclass = 'text-muted'
        elif obj.is_deletion():
            icon = '<i class="fa fa-trash"></i>'
            cssclass = 'text-danger'
        else:
            icon = ''

        if obj.is_deletion() or not obj.get_admin_url():
            link = obj.object_repr
        else:
            link = '<a href="%s">%s</a>' % (obj.get_admin_url(), obj.object_repr)

        return mark_safe('<span class="%s">%s %s</span>' % (cssclass, icon, link))

    repr_display.short_description = _('Entity')

    def content_display(self, obj):
        if obj.content_type:
            return capfirst(obj.content_type.name)
        else:
            return _('Unknown content')

    content_display.admin_order_field = 'content_type'
    content_display.short_description = _('Content')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        raise PermissionDenied

    def delete_view(self, request, object_id, extra_context=None):
        raise PermissionDenied

    def history_view(self, request, object_id, extra_context=None):
        raise PermissionDenied

    def get_urls(self):
        urlpatterns = []
        return urlpatterns


def context_data_for_app_index(func):
    @wraps(func)
    def _wrapper(*args, **kwargs):
        response = func(*args, **kwargs)
        response.context_data['app_label'] = kwargs.get('app_label')
        response.context_data['is_app_index'] = True
        response.context_data['admin_root_url'] = reverse('admin:index')
        return response
    return _wrapper

admin.site.__class__.app_index = context_data_for_app_index(admin.site.__class__.app_index)
